             .title        "JF-11/12/14 (INES 140) Reader Plugin"


;05/08/07 
;Quick mod of GNROM to use $6000 instead of $8000
;Version 1.0

;11/02/09
;BootGod
;JF-14 has an additional LS32 which gives it the ability to access up to
;128K CHR instead of just 32K. Rewrote CHR portion of code and increased the
;size detection range accordingly.

             
             ;vectors for standard system calls

send_byte:   .equ 0200h
baton:       .equ 0203h
chk_vram:    .equ 0206h
chk_wram:    .equ 0209h
wr_ppu:      .equ 020ch
read_byte:   .equ 020fh
init_crc:    .equ 0212h
do_crc:      .equ 0215h
finish_crc:  .equ 0218h

crc0:        .equ 0080h
crc1:        .equ 0081h
crc2:        .equ 0082h
crc3:        .equ 0083h

temp1:       .equ 00e0h
temp1_lo:    .equ 00e0h
temp1_hi:    .equ 00e1h
temp2:       .equ 00e2h
temp2_lo:    .equ 00e2h
temp2_hi:    .equ 00e3h

tx:          .equ 00dfh
curbank:    .equ 00e4h
banksleft:    .equ 00e5h


numpages_lo .equ 00e6h
numpages_hi .equ 00e7h
bank0crc:  .equ 00e8h
temp_crc:    .equ 00ech

;plugin header defines
p_ver: = 2	;plugin version
p_read: = 1	;plugin is for downloading (dumping)
p_write: = 2	;plugin is for uploading (RAM carts / WRAM etc)
p_other: = 3	;plugin is for some other purpose (i.e. BankWatch)
p_var1: = 4	;plugin uses user variable #1
p_var2: = 8	;plugin uses user variable #2
p_var3: = 16	;plugin uses user variable #3
p_var4: = 32	;plugin uses user variable #4

;plugin header data

             .org 0380h
             
             ;plugin description
             .db "Jaleco JF-11/12/14 carts"
             .fill 03e0h-*,00h

             ;plugin creator(s)
             .db "BootGod"
             .fill 03f8h-*,00h

             ;PRG/CHR/WRAM min/max sizes supported
             .db 32 >> 4,128 >> 4
             .db 16 >> 3,128 >> 3
             .db 0,0

             ;plugin version and flags
             .db p_ver
             .db p_read

;plugin code start here
             
             ldx #00h
             
             lda #020h
             jsr wr_ppu
             lda #055h
             sta 2007h
             lda #0aah
             sta 2007h
             lda #024h
             jsr wr_ppu
             lda 2007h
             lda 2007h
             cmp #055h
             bne horz_mir
             lda 2007h
             cmp #0aah
             bne horz_mir
             lda #020h
             jsr wr_ppu
             lda #0aah
             sta 2007h
             lda #024h
             jsr wr_ppu
             lda 2007h
             lda 2007h
             cmp #0aah
             bne horz_mir
             inx

horz_mir:    txa
             jsr send_byte   ;send byte
             
             lda #000h       ;get CRC of first bank
             jsr wr_bank
             sta temp1_lo
             lda #080h
             sta temp1_hi
             
             jsr init_crc
             ldy #0

chk_loop:    lda (temp1),y
             jsr do_crc
             iny
             bne chk_loop
             inc temp1_hi
             bne chk_loop   ;CRC all 32K
             jsr finish_crc
             ldx #3

ld_crc:      lda crc0,x
             sta temp_crc,x
             dex
             bpl ld_crc     ;store CRC
             jsr init_crc   ;crc for next block
             lda #010h       ;get CRC of 3rd bank
             jsr wr_bank
             lda #0
             sta temp1_lo
             lda #080h
             sta temp1_hi
             
             jsr init_crc
             ldy #0

chk_loop2:   lda (temp1),y
             jsr do_crc
             iny
             bne chk_loop2
             inc temp1_hi
             bne chk_loop2   ;CRC all 32K
             jsr finish_crc
             ldy #1
             ldx #3

chk_crc:     lda crc0,x             
             cmp temp_crc,x
             bne crc_fail
             dex
             bpl chk_crc
             jmp got_64k

crc_fail:    lda #020h       ;get CRC of 5th bank
             jsr wr_bank
             lda #0
             sta temp1_lo
             lda #080h
             sta temp1_hi
             
             jsr init_crc
             ldy #0

chk_loop3:   lda (temp1),y
             jsr do_crc
             iny
             bne chk_loop3
             inc temp1_hi
             bne chk_loop3   ;CRC all 32K
             jsr finish_crc

             ldy #4
             ldx #3

chk_crc2:    lda crc0,x             
             cmp temp_crc,x
             bne got_64k
             dex
             bpl chk_crc2
             dey             ;if the same, it's 128K else it's 256K
             dey

got_64k:     tya
             lsr a
             pha
             lda #0
             ror a
             jsr send_byte
             pla
             jsr send_byte
             lda #01h
             jsr send_byte   ;send our header

             tya             ; # banks to send
             tax
             lda #0
             sta temp2_lo

send_bank:   lda temp2_lo
             jsr wr_bank      ;set bank
             lda #010h
             clc
             adc temp2_lo
             sta temp2_lo    ;inc bank
             lda #0
             tay
             sta temp1_lo
             lda #080h
             sta temp1_hi

send_blk:    lda (temp1),y
             jsr send_byte
             iny
             bne send_blk
             jsr baton
             inc temp1_hi
             bne send_blk   ;send 32K
             dex
             bne send_bank

             
;determine CHR ROM size (16K - 128K)
             lda #0
             jsr wr_bank	;set to CHR bank 0
             jsr crc_cbank

             ldx #3
ld_ccrc:     lda temp_crc,x	;save for comparision
             sta bank0crc,x
             dex
             bpl ld_ccrc
             
;start out crc check at bank 2
             lda #2
             sta curbank

             lda #0		;pages init at 16K
             sta numpages_hi
             lda #040h
             sta numpages_lo

next_ccrc:   lda curbank
             jsr wr_bank	;set CHR bank num
             jsr crc_cbank
             
;compare crc's of bank 0 and curbank
             ldx #3
chk_ccrc:    lda bank0crc,x
             cmp temp_crc,x
             bne crc_cfail
             dex
             bpl chk_ccrc

             jmp got_chr

crc_cfail:   lda curbank	;check to see if we've hit the max bank num (16)
             cmp #16
             beq got_chr	;stop detection if we have

             asl curbank
             asl numpages_lo
             rol numpages_hi
             
             jmp next_ccrc
             
got_chr:     lda curbank	;setup bank counters for dumping
             sta banksleft
             lda #0
             sta curbank

             lda numpages_lo	;send CHR header
             jsr send_byte
             lda numpages_hi
             jsr send_byte
             lda #002h
             jsr send_byte
             
next_chr:    lda curbank
             jsr wr_bank
             inc curbank
             lda #0
             jsr wr_ppu
             lda 2007h

             ldx #32
             ldy #0

dump_chr:    lda 2007h
             jsr send_byte
             iny
             bne dump_chr
             dex
             bne dump_chr
             dec banksleft
             bne next_chr     
             
             
;send termination block
done:        lda #000h
             jsr send_byte
             lda #000h
             jsr send_byte
             lda #000h
             jsr send_byte
             rts

;sub for writing to bank reg
wr_bank:     sta 6000h
             rts


;CRC routine for 8K CHR bank @ $0000, stores crc in temp_crc
crc_cbank:   lda #0
             jsr wr_ppu
             lda 2007h		;set PPU address to $0000

             jsr init_crc
             ldx #32
             ldy #0		;init crc for 8K of data (32 pages = 8K)

crc_cloop:   lda 2007h		;crc generation loop
             stx tx
             jsr do_crc
             ldx tx
             iny
             bne crc_cloop
             dex
             bne crc_cloop
             
             jsr finish_crc	;crc done

             ldx #3
crc_csave:   lda crc0,x		;save crc in temp_crc
             sta temp_crc,x
             dex
             bpl crc_csave

             rts

             .fill 0800h-*,0ffh   ;fill rest to get 1K of data

             .end
